::xo::library doc {
  CMI Datatypes
}

namespace eval ::xo {}
namespace eval ::xo::scorm {}
namespace eval ::xo::scorm::data_model {}
namespace eval ::xo::scorm::data_model::data_types {

  Class DataType -superclass ::xotcl::Class -ad_doc {
    SCORMs CMI DataTypes
  }

  DataType create AbstractDataType \
      -parameter {value}

  AbstractDataType instproc init {args} {
    if {![[self class] is_valid_value [my set value]]} {
      error "Not valid input to construct DataType"
    }
  }

  AbstractDataType instproc to_string {} {
    return [my value]
  }

  AbstractDataType proc is_valid_value {value} {
    return true
  }

  DataType create CMIBlank -superclass AbstractDataType
  DataType create CMIBoolean -superclass AbstractDataType
  DataType create CMIDecimal -superclass AbstractDataType

  CMIDecimal proc is_valid_value {value} {
    if {[string is double -strict $value]
        && ![string match {-*} $value] } {
      return true
    }
    return false
  }

  DataType create CMIFeedback -superclass AbstractDataType
  DataType create CMIIdentifier -superclass AbstractDataType
  DataType create CMIInteger -superclass AbstractDataType
  DataType create CMIString255 -superclass AbstractDataType

  CMIString255 proc is_valid_value {value} {
    return [expr {[string length $value] <= 255 ? true : false}]
  }

  DataType create CMIString4096 -superclass AbstractDataType

  CMIString4096 proc is_valid_value {value} {
    set answer [expr {[string length $value] <= 4096 ? true : false}]
    #my log "$value is shorter than 4097? ... $answer"
    return $answer
  }

  DataType create CMITime -superclass AbstractDataType

  DataType create CMITimespan -superclass AbstractDataType -parameter {
    {hours 0000}
    {minutes 00}
    {seconds 00.00}
  }
  CMITimespan instproc init {args} {
    next
    #my log "args: $args, value: [my set value]"
    if {[my set value] ne ""} {
      set timespan [split [my set value] :]
      my hours [lindex $timespan 0]
      my minutes [lindex $timespan 1]
      my seconds [lindex $timespan 2]
    }
  }

  CMITimespan instproc add {cmitimespan} {
    if {![$cmitimespan istype CMITimespan]} {
      error "No CMITimespan"
    }
    my instvar hours minutes seconds
    my seconds [expr {[$cmitimespan seconds] + $seconds}]
    if {$seconds > 59} {
      set full_seconds [lindex [split $seconds .] 0]
      set deci_seconds [lindex [split $seconds .] 1]
      set rest [expr $full_seconds - 60]
      set minutes [expr $minutes + 1]
      set seconds ${rest}.${deci_seconds}
    }
    my minutes [expr [my minutes] + [$cmitimespan minutes]]
    if {$minutes > 59} {
      set rest [expr $minutes - 60]
      set hours [expr $hours + 1]
      set minutes $rest
    }
    my hours [expr [my hours] + [$cmitimespan hours]]
    return [self]
  }

  CMITimespan instproc to_string {} {
    my instvar hours minutes
    set secs [my set seconds]
    if {![string match *.* $secs] && ![string match 00 $secs]} {
      # It seems as if we have no comma, this might lead to
      # "second values" like 08, which may lead to TCL errors because
      # of octal value interpretation (see http://wiki.tcl.tk/498)
      set secs [string trimleft $secs 0]
    }
    set t "[format %02d $hours]:[format %02d $minutes]:[format %05.2f $secs]"
    my log $t
    return $t
  }

  CMITimespan proc is_valid_value {value} {
    # Valid is HHHH:MM:SS.SS
    return [regexp {^[0-9]{2,4}:[0-9]{2}:[0-9]{2}(\.[0-9]{1,2})?$} $value]
  }



  Class create CMIVocabulary -superclass DataType \
    -parameter {
      value
      allowed_values
    }

  CMIVocabulary instproc is_valid_value {value} {
    #my log "allowed are: [my allowed_values]"
    if {[lsearch -exact [my allowed_values] $value] == -1} {
      return false
    }
    return true
  }

  CMIVocabulary create Mode -superclass AbstractDataType \
    -allowed_values {
      "normal"
      "review"
      "browse"
    }

  CMIVocabulary create Status -superclass AbstractDataType \
    -allowed_values {
      "passed"
      "completed"
      "failed"
      "incomplete"
      "browsed"
      "not attempted"
    }

  CMIVocabulary create Exit -superclass AbstractDataType \
    -allowed_values {
      ""
      "time-out"
      "suspend"
      "logout"
    }

  CMIVocabulary create Credit -superclass AbstractDataType \
    -allowed_values {
      "credit"
      "no-credit"
    }

  CMIVocabulary create Entry -superclass AbstractDataType \
    -allowed_values {
      "ab-initio"
      "resume"
      ""
    }

  CMIVocabulary create Interaction -superclass AbstractDataType \
    -allowed_values {
      "true-false"
      "choice"
      "fill-in"
      "matching"
      "performance"
      "likert"
      "sequencing"
      "numeric"
    }

  CMIVocabulary create Result -superclass AbstractDataType \
    -allowed_values {
      "correct"
      "wrong"
      "unanticipated"
      "neutral"
      CMIDecimal
    }

  CMIVocabulary create TimeLimitAction -superclass AbstractDataType \
    -allowed_values {
      "exit,message"
      "exit,no message"
      "continue,message"
      "continue,no message"
    }

  namespace export *

}

::xo::library source_dependent
