::xo::library doc {

    SCORM Support for XoWiki

    @see http://www.imsglobal.org/content/packaging/
    @creation-date 2009
    @author Michael Aram
}

# TODO: Enforce the suffix .html when pages are created!!! We need that
# TODO: Automatically create xoscorm-index.html page (based on prototype)
# TODO: Do we want to add .html to all pages?

::xo::library require -package xowiki xowiki-procs
::xo::library require -package xolrn package-procs
::xo::library require xml-procs
::xo::library require scorm-cam-procs

namespace eval ::xo {}
namespace eval ::xo::scorm {

  ::xo::PackageMgr create ::xo::scorm::Package \
    -package_key "xoscorm" \
    -pretty_name "XOSCORM" \
    -table_name "xoscorm" \
    -superclass ::xowiki::Package

  Package ad_instproc initialize {} {
    Upon initialization, we mixin our "enhancements" into the respective base classes
  } {
    next
    #::xo::scorm::Package instmixin add ::xolrn::Package
  }

  if {[info command ::xolrn::Package] ne ""} {
    ::xolrn::Package instmixin add ::xo::scorm::Package
    ns_log notice "SCORM functionality activated in XOLRN"
  } else {
    ns_log notice "XOLRN Package class not found - SCORM functionality deactivated"
  }


  Package instproc item_info_from_url {{-with_package_prefix true} {-default_lang ""} url} {
    #
    #  Check which SCO the client wants to launch
    #
    set sco_id [::xo::cc query_parameter xoscorm_sco_id]
    array set "" [next]
    if {$sco_id ne ""} {
      #my msg "\n\n\nPackage launches: Trying to launch course '$(item_id)' and sco '$sco_id'\n\n\n"
      ::xo::scorm::session launch -course_id $(item_id) -sco_id $sco_id
    } else {
      #my log "Nothing to launch for sco_id: '$sco_id'"
    }
    return [array get ""]
  }

  Package ad_instproc require_xoscorm_folder {
    {-course_id:required}
  } {
    @author Michael Aram
  } {
    if {$course_id eq ""} {error "course_id required"}
    set xoscorm_folder_name ".xoscorm"
    set course [::xo::db::CrClass get_instance_from_db -item_id $course_id]
    set course_package_id [$course package_id]
    #::xoscorm::Package require $course_package_id
    set item_id [::xo::db::CrClass lookup \
        -name $xoscorm_folder_name -parent_id $course_id]
    if {$item_id == 0} {
      set form_id [::xowiki::Weblog instantiate_forms \
          -forms en:folder.form -package_id [my id]]
      set default_vars [list title "XOSCORM Usage Data" \
          parent_id $course_id \
          package_id $course_package_id]
      set f [$form_id create_form_page_instance \
          -package_id $course_package_id \
          -name $xoscorm_folder_name \
          -nls_language en_US \
          -publish_status ready \
          -default_variables $default_vars]
      $f save_new
      my log "Created .xoscorm folder for course: $course_id"
      set item_id [$f item_id]
    }
    return $item_id
  }

  Package ad_instproc require_course_session_data {
    {-course_id:required}
  } {
    @author Michael Aram
  } {
    if {$course_id eq ""} {error "course_id required"}
    set parent_id [my require_xoscorm_folder -course_id $course_id]
    set xoscorm_folder [::xo::db::CrClass get_instance_from_db -item_id $parent_id]

    set session_form [my get_page_from_item_ref \
        -use_site_wide_pages false \
        -use_package_path true \
        -use_prototype_pages true \
        en:scorm_course_session.form ]

    set user_id [::xo::cc user_id]
    set name $user_id
    set title "[::xo::get_user_name $user_id] in course $course_id (SCORM Course Session Data)"
    set course_data_item_id [::xo::db::CrClass lookup -name $name -parent_id $parent_id]
    if {$course_data_item_id == 0} {
      set fp [$session_form create_form_page_instance \
          -name $name \
          -default_variables [list title $title] \
          -package_id [$xoscorm_folder package_id] \
          -parent_id $parent_id \
          -nls_language en_US \
          -publish_status "ready"]
      $fp save_new
      my log "Created $title"
      set course_data_item_id [$fp item_id]
    }
    if {[info command $course_data_item_id] eq ""} {
      ::xo::db::CrClass get_instance_from_db -item_id $course_data_item_id
    }
    ::xo::Package require [$course_data_item_id package_id]
    return $course_data_item_id
  }

  Package ad_instproc exists_course_session_data {
    {-course_id:required}
    {-user_id ""}
  } { } {
    if {$user_id eq ""} {set user_id [::xo::cc user_id]}
    set xoscorm_folder_id [my require_xoscorm_folder -course_id $course_id]
    set user_folder_id [::xo::db::CrClass lookup -name $user_id -parent_id $xoscorm_folder_id]
    return [expr {$user_folder_id == 0} ? false : true]
  }

  Package ad_instproc exists_sco_session_data {
    {-sco_id:required}
    {-course_id:required}
    {-user_id ""}
  } { } {
    if {$user_id eq ""} {set user_id [::xo::cc user_id]}
    set xoscorm_folder_id [my require_xoscorm_folder -course_id $course_id]
    if {![my exists_course_session_data -course_id $course_id]} {
      return false
    } else {
      set 
    }
    set user_folder_id [::xo::db::CrClass lookup -name $user_id -parent_id $xoscorm_folder_id]
    return [expr {$user_folder_id == 0} ? false : true]
  }

  Package ad_instproc require_sco_session_data {
    {-sco_id:required}
    {-course_id:required}
  } {
    Create or use the formpage which stores the cmi data model
    values for this user/course/lesson.
    @author Michael Aram
  } {
    if {$sco_id eq "" } {error "sco_id required"}
    if {$course_id eq ""} {error "$course_id are required"}

    # We use the Course Session Data FormPage as Container
    set course_session_data_fp [my require_course_session_data -course_id $course_id]
    set session_data_parent_id [$course_session_data_fp item_id]
    ::xo::Package require [$course_session_data_fp package_id]

    set session_form [my get_page_from_item_ref \
        -use_site_wide_pages false \
        -use_package_path true \
        -use_prototype_pages true \
        en:scorm_sco_session.form ]

    set user_id [::xo::cc user_id]
    set name SSD-${course_id}-${user_id}-$sco_id
    set title "SCORM Session Data for [::xo::get_user_name $user_id] in SCO $sco_id in course $course_id"
    set sco_data_fp_id [::xo::db::CrClass lookup -name $name -parent_id $session_data_parent_id]
    if {$sco_data_fp_id == 0} {
      set session_data_fp [$session_form create_form_page_instance \
          -name $name \
          -default_variables [list title $title] \
          -package_id [$course_session_data_fp package_id] \
          -parent_id $session_data_parent_id  \
          -nls_language en_US \
          -publish_status "ready"]
      $session_data_fp save_new
      my log "Created $title"
      set sco_data_fp_id [$session_data_fp item_id]
    }
    if {[info command $sco_data_fp_id] eq ""} {
      ::xo::db::CrClass get_instance_from_db -item_id $sco_data_fp_id
    }
    return $sco_data_fp_id
  }

    # TODO Clean this up - there are more beautiful functions now
    Package ad_instproc generate_manifest {} {
        Update the current existing manifest (or create one from scratch if it doesnt exists).
        Organizations coming from imported packages are preserved.
        Three "self-made" organizations are included:
        <ul>
        <li>Book Organization
        <li>Category Organization
        <li>Wiki Organization (offers just an entry to the wiki)
        </ul>
      } {
        set res [list]
        foreach fileobj [my get_wiki_content_as_fileobjects] {
            if {[$fileobj exists is_wikifile] && [$fileobj set name] != "imsmanifest.xml"} {
                set f [::ims::cp::InternalFile create [my autoname internalfile] -destroy_on_cleanup -href "download/file/[$fileobj get_cp_filename]"]
            } else {
                set f [::ims::cp::InternalFile create [my autoname internalfile] -destroy_on_cleanup -href "[$fileobj get_cp_filename]"]
            }
            # TODO this is only good for our wiki-orga and dirty
            if { [$f set href] eq "xoscorm-index.html" } {
                lappend res [::xo::scorm::Resource create xoscorm-index -adlcp:scormtype "sco" -destroy_on_cleanup -files $f]
            } else {
                set itemid [::xoutil::XMLClass get_valid_id [[$fileobj set critem] revision_id]]
                lappend res [::xo::scorm::Resource create [my autoname resource] -identifier $itemid -adlcp:scormtype "sco" -destroy_on_cleanup -files $f]
            }
        }

        set all_orgas [list]

        set current_orgas [[my get_manifest]::organizations children]

        # Add the Organizations, that we did not create ourselves (import), as they are.
        foreach orga $current_orgas {
            if {[regexp "xoscorm" [$orga identifier]]} { continue }
            lappend all_orgas $orga
        }


        foreach catorg [my get_category_organizations] {
            lappend all_orgas $catorg
        }
        lappend all_orgas [my get_wiki_organization]
        lappend all_orgas [my get_book_organization]

        ::xo::scorm::Manifest man -destroy_on_cleanup -resources $res -temp true -organizations $all_orgas

        man set package_id [my set id]
        man set folder_id  [my set folder_id]
        man mixin add ::xo::scorm::File

        # Check if there is a Manifest in the xowiki instance
        set critem [my get_page_from_name -name "file:imsmanifest.xml"]
        if {$critem ne ""} {
            $critem set import_file [man set location]
            $critem save
        } else {
            man to_xowiki
        }
    }

    Package ad_instproc get_category_organizations {} {
        returns the cat-tree as scorm org
      } {

        set package_id [my id]
        #FIXME The includelet requires a parent object (due to a bug(?), because it can also be a dummy object)
        ::xowiki::includelet::CategoryOrganization create [self]::cat_organization -package_id $package_id -set expand_all true
        [self]::cat_organization initialize
        set xml [[self]::cat_organization render]

        set doc [dom parse $xml]

        set catorgas [list]
        set counter 0
        foreach catorgdom [[$doc documentElement] childNodes] {
            lappend catorgas [::xo::scorm::Organization new -identifier "xoscorm-cat-org-$counter" -dom $catorgdom]
            incr counter
        }
        return $catorgas
    }

    Package ad_instproc get_wiki_organization {} {
        returns a simple SCORM Organization, that contains only an entry page sco
      } {
        set ::wikiname [my set instance_name]

        ::xo::scorm::Organization create [self]::wiki_organization -identifier "xoscorm-wiki-org" -contains {
            ::ims::cp::Title t -text "$::wikiname"
            ::ims::cp::Item xoscorm-item-initialize -identifierref "xoscorm-index" -contains {
                ::ims::cp::Title t1 -text "Enter Wiki"
            }
        }

        return [self]::wiki_organization
    }

    Package ad_instproc get_book_organization {} {
        returns the book-toc as scorm org
      } {

        set package_id [my id]
        #FIXME The includelet requires a parent object (due to a bug(?), because it can also be a dummy object)
        ::xowiki::includelet::BookOrganization create [self]::book_organization -package_id $package_id -set expand_all true
        [self]::book_organization initialize
        #[self]::book_organization set id "book[my id]"
        #[self]::book_organization build_navigation [[self]::book_organization build_toc $package_id en_US "" ""]
        #set xml [[self]::book_organization render -full true [[self]::book_organization build_toc $package_id en_US "" ""]]
        set xml [[self]::book_organization render]

        set doc [dom parse $xml]

        ::xo::scorm::Organization create [self]::book_org -identifier "xoscorm-book-org" -dom [$doc documentElement]

        return [self]::book_org
    }

    Class LinkRewriter
    LinkRewriter instproc pretty_link args {
        # we need a relative URL
        set package_prefix "[my set package_url]"
        set absolute_internal_url [next]
        set relative_url [regsub $package_prefix $absolute_internal_url ""]
        return $relative_url
    }

    Package ad_instproc get_page_content {cr_item} {
      This method gets the contents of the page, but in contrast to normal xowiki "rendering"
      it changes the link-generation process so that it generates relative urls.
      Furthermore, it tries to detect, whether the page was originally imported, which means it
      contains a "full html page", or whether it is an ordinary xowiki page, which only contains
      an html fragment. In the latter case, the page content is wrapped.
      } {
        my mixin add ::xo::scorm::LinkRewriter
        set content ""
        # Fill the file with content using "render"
        set content [$cr_item render -with_footer false]

            # Using "view" method could be an option?
            #::xo::ConnectionContext require -url [ad_conn url]
            #::xo::cc set_parameter template_file "/packages/xowiki-ims-cp/www/view-sco"
            #$f set content [$cri view]
        my mixin delete ::xo::scorm::LinkRewriter
        return $content
    }


    Package ad_instproc export_wiki_as_cp {} {
        This is the trigger function for exporting a wiki instance as content package.
      } {
        my generate_manifest
        set fileobjects [my get_wiki_content_as_fileobjects]
        #ds_comment "[my serialize]"
        set zipfilename [regsub {/$} [my set package_url] ""]
        #ContentPackage create [self]::cp -location "[acs_root_dir]/packages/ims-cp[ns_tmpnam]$zipfilename"
        ContentPackage create [self]::cp -location "[ns_tmpnam]$zipfilename"
        # Create folders for xowikis "proprietary" "download/file" links
        ::xoutil::Folder create [self]::cp::download -name "download" -parent_folder [self]::cp
        ::xoutil::Folder create [self]::cp::download::file -name "file" -parent_folder [self]::cp::download
        foreach fileobject $fileobjects {
            if {[$fileobject exists is_wikifile] && [$fileobject set name] != "imsmanifest.xml"} {
                [self]::cp::download::file add_file $fileobject
            } else {
                [self]::cp add_file $fileobject
            }
        }
        [self]::cp add_file [::xoutil::File create getapi -location "[acs_root_dir]/packages/scorm/www/resources/1.2/get_api.js"]
        set pif [[self]::cp pack]
        $pif deliver
    }


    Package ad_instproc get_childpages {-crfolder_id} {
        #TODO
    } {
        set sql [::xowiki::Page instance_select_query -with_subtypes 1 -folder_id $crfolder_id -with_children false]
        set pages [list]
        db_foreach retrieve_instances $sql {
             # not the folder object 
             if {[regexp {::.*} $name]} { continue }
             # Omit the folder-formpages
             set page [::xo::db::CrClass get_instance_from_db -item_id $item_id]
             if {[$page istype ::xowiki::FormPage] && [[$page get_template_object] name] eq "en:folder"} {
               #
             } else {
               lappend pages $page
             }
        }
        return $pages
    }

    Package ad_instproc get_subfolders {-crfolder_id} {
        #TODO
    } {
        my instvar id
        set folder_form_id [::xowiki::Weblog instantiate_forms \
                               -forms en:folder.form \
                               -package_id $id]
        set folder_oc [::xowiki::FormPage get_form_entries \
                              -parent_id $crfolder_id \
                              -base_item_ids $folder_form_id -form_fields "" \
                              -publish_status ready -package_id $id]
        set folders [$folder_oc children]
        return $folders
    }

    Package ad_instproc export_scorm_cp {} {
        new test
      } {
        set zipfilename [regsub {/$} [my set package_url] ""]
        set cp [ContentPackage create [self]::cp -location "[ns_tmpnam]$zipfilename"]

        my add_pages_to_fs_folder -crfolder_id [my folder_id] -fsfolder $cp

        foreach crfolder "[my get_subfolders -crfolder_id [my folder_id]]" {
            my build_fsfolder -cr_folder $crfolder -parent_folder $cp
        }
        set pif [[self]::cp pack]
        $pif deliver
    }

    Package ad_instproc build_fsfolder {-cr_folder -parent_folder} {
        Create a folder for the content package / ZIP file.
        FIXME: Note, that we strip the language prefix, because windows
        doesnt support ":" in filenames.
      } {
        my get_lang_and_name -name [$cr_folder name] lang stripped_name
        set f [::xoutil::Folder new -name $stripped_name -parent_folder $parent_folder]
        my log "[$f serialize]"
        my add_pages_to_fs_folder -crfolder_id [$cr_folder item_id] -fsfolder $f
        foreach sub_crfolder "[my get_subfolders -crfolder_id [$cr_folder item_id]]" {
            my build_fsfolder -cr_folder $sub_crfolder -parent_folder $f
        }
    }

    Package ad_instproc add_pages_to_fs_folder {-crfolder_id -fsfolder} {} {
        foreach page [my get_childpages -crfolder_id $crfolder_id] {
            set f [File new -temp true]
            if {[$page istype ::xowiki::File]} {
                $f set location "[$page full_file_name]"
                $f set name [regsub "^file:" [$page set name] ""]
                $f set mime_type [$page mime_type]
            } elseif {[$page istype ::xowiki::Page]} {
                $f set location [ns_tmpnam]
                $f set name [regsub "^..:" [$page set name] ""]
                $f set mime_type [$page mime_type]
                $f set content [$page render]
                $f save
            } else {
                ds_comment "[$f serialize]"
            }
            $fsfolder add_file $f
        }
    }

    Package ad_instproc get_wiki_content_as_fileobjects {} {
        TODO: CLEANUP
      } {
        # Get all Pages
        set pageids [list]
        set sql [::xowiki::Page instance_select_query -with_subtypes 1 -folder_id [my set folder_id] -with_children true]
        db_foreach retrieve_instances $sql {
             # not the folder object 
             if {[regexp {::.*} $name]} {
                 continue
             }
             lappend pageids $item_id
        }


                        set folder_attributes [list creation_user title parent_id label \
                                            "to_char(last_modified,'YYYY-MM-DD HH24:MI:SS') as last_modified" ]
                        set folder_id [my folder_id]

                        set folder_sql [::xo::db::CrFolder instance_select_query \
                                 -folder_id $folder_id \
                                 -select_attributes $folder_attributes \
                                 -with_subtypes true \
                                 -with_children false \
                                 -orderby ci.name \
                                ]

                        db_foreach folder_select $folder_sql {
                                my msg "name: $name"
                        }



        set files [list]

        foreach item_id $pageids {
            set cri [::xo::db::CrClass get_instance_from_db -item_id $item_id]
            if {[file isfile "[acs_root_dir]/content-repository-content-files[$cri set text]"]} {
                # The actual file
                set f [File create [my autoname wikifile] -location "[acs_root_dir]/content-repository-content-files[$cri set text]"]
                $f set name [regsub "^file:" [$cri set name] ""]
                $f set mime_type [$cri set mime_type]
                # MARK THOSE THAT SHOULD GO INTO download/file
                $f set is_wikifile true
                # associate the critem for convenience
                $f set critem $cri
                #$f set_extension_based_on_mimetype
                lappend files $f

                # The file overview page
                #set f [::xoutil::TempFile create [my autoname exportfile]]
                #$f set name [regsub "^file:" [$cri set name] ""]
                #lappend files $f
            } else {
                #TODO do we still/really need that?
                $cri set absolute_links 0
                set f [File create [my autoname exporttmpfile] -temp true ]
                $f set critem $cri
                $f set mime_type [$cri set mime_type]
                $f set name [regsub "^en:" [$cri set name] ""]

                $f set content [my get_page_content $cri]

                $f save
                lappend files $f
            }
        }
        return $files
    }

    Package ad_instproc get_page_for_item {item} {
        Returns 
      } {
        my instvar id
        set href [my get_href_for_item -item $item]
        my log "Item '$item' has href '$href'"
        set page [expr {$href eq "" ? "" : [my resolve_page $href view]}]
        if {[$item exists parameters]} {
          # We have an item with URL parameters, thus we search for the redirect formpage
          # instead of the target (= page without parameters)
          set target_url [my pretty_link -parent_id [$page parent_id] [$page name]]
          append target_url [$item set parameters]
          # Get the redirect form
          set redirect_form [::xowiki::Weblog instantiate_forms \
                               -forms en:redirect \
                               -package_id $id]
          # Get all instances
          set redirects [::xowiki::FormPage get_form_entries \
                              -base_item_ids [$redirect_form item_id] -form_fields "" \
                              -publish_status ready -package_id $id]

          foreach candidate [$redirects children] {
              my log "AHB [$candidate serialize]"
            # todo : this will not work for 2 items with same target
            if {[$candidate exists __ia(redirect_url)] && [$candidate set __ia(redirect_url)] eq $target_url} {
              set page $candidate
              my msg "REDIRECT $candidate"
            }
          }
        }
        return $page
    }

    # Deprecated: Use item->href (why did I loop over the resources here?)
    Package instproc get_href_for_item {-item} {
        set m [my get_manifest]
        # Get the Resource attached to the Item, if any
        if {[$item exists identifierref]} {
            set r [$m get_element_by_identifier [$item set identifierref]]
            foreach f [${r}::files children] {
                if {[$r set href] eq [$f set href]} {
                    set base_href "[$m set xml:base]"
                    return $base_href[$r set href]
                }
            }
        }
        return ""
    }

    Package ad_instproc post_upload {} {} {
        if {![my has_manifest]} {
            util_user_message -message "#xoscorm.no_manifest_found#"
            return ""
        }
        # First, resolve dependencies
        my resolve_dependencies
        #my generate_redirects
        my decorate_titles
        my decorate_page_order
        my generate_category_organization
    }

    Package ad_instproc generate_category_organization {} {
        Gen Cat based on org
      } {
        # TODO import all - not only the default (easy)
        set m [my get_manifest]
        set o [$m get_default_or_implicit_organization]

        set cat_tree_id [category_tree::add -name "[$o title]" -description "SCORM Organization of Wiki [my id]" -context_id [my id]]
        category_tree::map -tree_id $cat_tree_id -object_id [my id]
        $o generate_category -cat_tree_id $cat_tree_id
        my resolve_dependencies

        my map_items_to_categories
    }

    Package ad_instproc map_items_to_categories {} {
        maps the page associated with the file, which is referenced by a resource that is referenced by an item :-) to a category
      } {
        set m [my get_manifest]
        set o [$m get_default_or_implicit_organization]

        foreach item [$o all_children_of_type Item] {
            set page [my get_page_for_item $item]
            if {$page ne ""} {
                my log "Mapping [$page name] to category [$item set __item_cat_id]"
                category::map_object -object_id [$page item_id] [$item set __item_cat_id]
            }
        }
    }

    Package ad_instproc generate_redirects {} {
        We have one page per resource. Thus, if we have more items pointing to one
        resource, we generate redirects, that point to that resource.
      } {
        my instvar id
        set m [my get_manifest]
        set o [$m get_default_or_implicit_organization]
        set redirect_form [::xowiki::Weblog instantiate_forms \
                               -forms en:redirect \
                               -package_id $id]

        foreach item [$o all_children_of_type Item] {
            set target_page [my get_page_for_item $item]
            if {$target_page ne "" && [$item exists parameters]} {
                # TODO: is target_page parent_id always sensible??
                set target_url [my pretty_link -parent_id [$target_page parent_id] [$target_page name]]
                append target_url [$item set parameters]
                set redirect_name en:redirect_to_[my autoname [$target_page item_id]]
                set redirect_id [::xo::db::CrClass lookup -name $redirect_name -parent_id [$target_page parent_id]]
                if {$redirect_id == 0} {
                  set fp [$redirect_form create_form_page_instance \
                             -name $redirect_name \
                             -nls_language en_US \
                             -publish_status "ready" \
                             -creation_user "MISCHI" \
                             -instance_attributes [list title [$item title] \
                                                      parent_id [$target_page parent_id] \
                                                      creator "MISCHI" \
                                                      redirect_url $target_url \
                                                      package_id $id]]
                  $fp set creator "Mischa"
                  my log "Creating Redirect-FormPage to '$target_page'"
                  $fp save_new
                } else {
                  # TODO : This is unfinished
                  my log "Creating Revision of Redirect-FormPage to '$target_page'"
                  set fp [::xo::db::CrClass get_instance_from_db -item_id $redirect_id]
                  $fp set name $redirect_name
                  $fp set creator "Mischa"
                  $fp save
                }
            }
        }
    }

    Package ad_instproc resolve_dependencies {} {
        Replace all Dependency instances in the composite with their associated resources
      } {
        set m [my get_manifest]
        foreach r [${m}::resources children] {
            # Get the required Resource
            foreach d [${r}::dependencies children] {
                set res [$m get_element_by_identifier [$d set identifierref]]
                foreach depfile [${res}::files children] {
                    ${r}::files add $depfile
                }
            }
        }
    }

    Package ad_instproc decorate_titles {} {
        Decorate the pages inside this instance with titles from manifest items
      } {
        set m [my get_manifest]
        set o [$m get_default_or_implicit_organization]

        foreach item [$o all_children_of_type Item] {
            set page [my get_page_for_item $item]
            if {$page ne ""} {
                my log "Decorating page '[$page name]' with title '[$item title]'"
                $page set title [$item title]
                $page set creator [::xo::cc user]
                $page save
            }
        }
    }

    Package ad_instproc decorate_page_order {} {
        Decorate the pages inside this instance with page_order values according to the manifest file
      } {
        set m [my get_manifest]
        set o [$m get_default_or_implicit_organization]
        $o decorate_page_order

        # First, resolve dependencies

        my resolve_dependencies
        foreach item [$o all_children_of_type Item] {
            set page [my get_page_for_item $item]
            if {$page ne ""} {
                my log "Decorating [$page name]  $item "
                $page set page_order [$item set __page_order]
                $page save
            }
        }
    }
}
::xo::library source_dependent
