::xo::library doc {
  SCORM CAM specific extensions to IMS CP
}

::xo::library require ims-cp-procs

package require tdom
namespace eval ::xo {}
namespace eval ::xo::scorm {}
namespace eval ::xo::scorm::xml {

  dom createNodeCmd cdataNode   cdata
  dom createNodeCmd textNode    t
  dom createNodeCmd commentNode c
  dom createNodeCmd parserNode  x
  dom createNodeCmd piNode      runtime

  dom createNodeCmd elementNode manifest
  dom createNodeCmd elementNode metadata
  dom createNodeCmd elementNode schema
  dom createNodeCmd elementNode schemaversion
  dom createNodeCmd elementNode organizations
  dom createNodeCmd elementNode organization
  dom createNodeCmd elementNode title
  dom createNodeCmd elementNode item
  dom createNodeCmd elementNode resources
  dom createNodeCmd elementNode resource
  dom createNodeCmd elementNode file
  dom createNodeCmd elementNode dependency

}
namespace eval ::xo::scorm {

    Class Manifest -superclass ::ims::cp::Manifest -parameter {
        {version "1"}
    }

    Manifest instproc asXML {} {
        set document [dom createDocumentNode]
        $document appendFromScript {
        [my tdomcmd] \
            -identifier [my identifier] \
            -xmlns "http://www.imsproject.org/xsd/imscp_rootv1p1p2" \
            -xmlns:adlcp "http://www.adlnet.org/xsd/adlcp_rootv1p2" \
            -xmlns:xsi "http://www.w3.org/2001/XMLSchema-instance" \
            -xsi:schemaLocation "http://www.imsproject.org/xsd/imscp_rootv1p1p2 imscp_rootv1p1p2.xsd http://www.imsglobal.org/xsd/imsmd_rootv1p2p1 imsmd_rootv1p2p1.xsd http://www.adlnet.org/xsd/adlcp_rootv1p2 adlcp_rootv1p2.xsd" \
        }

        [$document documentElement] appendFromScript {
            [self]::metadata asXML
            [self]::organizations asXML
            [self]::resources asXML
        }

        set xml [$document asXML]
        $document delete

        return $xml
    }

    Class ContentPackage -superclass ::ims::cp::ContentPackage


    ContentPackage instproc init args {
        next
        # IMPORT Standard files
        set std_files {
            adlcp_rootv1p2.xsd
            imscp_rootv1p1p2.xsd
            imsmd_rootv1p2p1.xsd
            ims_xml.xsd
        }

        foreach std_file $std_files {
            my add_file [::xoutil::File create [my autoname static] -location "[acs_root_dir]/packages/xoscorm/www/resources/1.2/$std_file"]
        }
    }



    Class ResourcePackage -superclass ContentPackage
    Class ContentAggregationPackage -superclass ContentPackage

    Class Resource -superclass ::ims::cp::Resource -parameter {
        {adlcp:scormtype "sco"}
    }

    Resource instproc asXML {} {
        set files_comp [self]::files
        set files_list [$files_comp set __children]
        # If we contain only one single file, then our "entry point" is clear, thus we set
        # the href to the href of this single file.
        if {![my exists href] && [llength $files_list] eq 1 && [[lindex $files_list 0] exists href]} {
            my set href [[lindex $files_list 0] set href]
        }


        [my tdomcmd] [my get_attributes identifier type xmlbase href adlcp:scormtype] {
                foreach f [$files_comp children] {
                    $f asXML
                }
        }
    }

}

::xo::library source_dependent
