namespace eval ::xo {}
namespace eval ::xo::scorm {
  ###################
  #                 #
  #  SCORM SESSION  #
  #                 #
  ###################

  Object session

  session ad_proc launch {-course_id -sco_id} {
    Launch a new SCO. Take care of persisting the old current SCO
    @sco_id The XML id of the SCO as in the manifest.
  } {
    my log "\n\nLaunching $course_id $sco_id \n\n"
    #if {[my get_current_course_id] ne $course_id} {
      my set_current_course_id -course_id $course_id
    #}
    #if {[my get_current_sco_id] ne $sco_id} {
      my set_current_sco_id -sco_id $sco_id
    #}
  }

  #
  # Course Level
  #

  session ad_proc set_current_course_id {-course_id} {} {
    return [ad_set_client_property "xoscorm" "current_course" $course_id]
  }

  session ad_proc get_current_course_id {} {} {
    return [ad_get_client_property "xoscorm" "current_course"]
  }

  session ad_proc set_current_sco_id {-sco_id} {} {
    set package_id [::xo::cc package_id]
    set course [::xo::db::CrClass get_instance_from_db -item_id [my get_current_course_id]]
    set fp [$package_id require_course_session_data -course_id [$course item_id]]
    $fp set_property -new true current_sco $sco_id
    my log "Persisting course session data ([$fp title])"
    $fp save
  }

  session ad_proc get_current_sco_id {} {} {
    set package_id [::xo::cc package_id]
    set course [::xo::db::CrClass get_instance_from_db -item_id [my get_current_course_id]]
    set fp [$package_id require_course_session_data -course_id [$course item_id]]
    return [$fp get_property -name current_sco]
  }


  #
  # SCO Level
  #

  session ad_proc commit {} {
    Persist the session data in the database.
  } {
    set session_fp [my get_sco_session_data]
    my log "Committing SCO session data $session_fp ([$session_fp title])"
    $session_fp save
  }

  session proc get_sco_session_data {{-course_id ""} {-sco_id ""}} {
    set package_id [::xo::cc package_id]
    ::xo::scorm::Package require $package_id
    if {$course_id eq ""} {
      set course_id [::xo::cc form_parameter xoscorm_course_id]
    }
    if {$sco_id eq ""} {
      set sco_id [::xo::cc form_parameter xoscorm_sco_id]
    }
    if {$course_id eq ""} {
      error "no course_id"
    }
    if {$sco_id eq ""} {
      error "no sco_id"
    }
    return [$package_id require_sco_session_data -course_id $course_id -sco_id $sco_id]
  }

  session proc get_property {property} {
    set session_fp [my get_sco_session_data]
    set property_value [$session_fp get_property -name $property]
    return $property_value
  }

  session proc set_property {property value} {
    set session_fp [my get_sco_session_data]
    $session_fp set_property -new true $property $value
    #$session_fp save
    $session_fp flush_from_cache_and_refresh
  }

  session ad_proc -private is_initialized {
  } {
    Check if the current sco is initialized
  } {
    if {[my get_property execution_state] eq "initialized"} {
      return true
    }
    return false
  }
}
