::xo::library doc {
  XOLRN - utility procs

  This file contains some utility functions mainly used by
  other packages like xowiki-ims-cp or scorm. We may consider
  to create a shared package (e.g. xotcl-lib), but as long as
  this is immature there is no good reason...

  @creation-date 2010-02-12
  @author Michael Aram
}

namespace eval ::xolrn {}
namespace eval ::xoutil {

  Object id_generator

  id_generator ad_proc get_id_for_string {name} {
    This method returns an id-string based upon a given string.
    The function is idempotent, i.e. for the same string, it always
    returns the same id.
    The advantage of this method over e.g. the "::xowiki::Includelet html_id"
    method is, that the id string can be used directly e.g. as a JavaScript
    identifier or as an XML id (HTML ID allow more chars than those).

    This is currently used for creating IDs for SCORM items
    @author Michael Aram
  } {
      package require md5
      return "ID[md5::md5 -hex \"$name\"]"
  }

  ###########
  #         #
  #   XML   #
  #         #
  ###########

  Class create XMLClass -superclass ::xotcl::Class -parameter {
    tdom_cmd_name
    tdom_attributes
  } -ad_doc {
    This is a meta-class for classes whose instances represent XML elements.
    Currently this is used by the QTI implementation.

    # TODO: Remove the stuff from init and move it to an additional class,
    #       and add this class as a mixin.
    @author Michael Aram
  }

  XMLClass instproc init args {
      my instmixin add ::xo::OrderedComposite
      if  {![my exists tdom_cmd_name]} {
          my set tdom_cmd_name [namespace tail [self]]
      }
      my set tdom_cmd "[self]::xml::[my set tdom_cmd_name]"

      # Provide for the actual instances an easy2use method,
      # which ensures existence of the tdom command and returns
      # the actual command
      my instproc tdomcmd {} {
          [self class] require_tdom_cmd
          return [[self class] set tdom_cmd]
      }

      my instproc get_tdom_attributes {} {
          set pairs [list]
          if  {[[self class] exists tdom_attributes]} {
              foreach att [[self class] tdom_attributes] {
                  lappend pairs $att [my set $att]
              }
          } else {
              # Export all instance variables that do not start with an underscore
              foreach att [my info vars {[a-zA-Z0-9]*}] {
                  #TODO Remove the dom hack here
                  if {$att ne "dom"} {
                      lappend pairs $att [my set $att]
                  }
              }
          }
          return $pairs
      }

      # default "asXML" method. may be overwritten
      my instproc asXML {} {
          if {[llength [my children]] == 0 && [my exists omit_if_no_children]} return
          [my tdomcmd] [my get_tdom_attributes] {
              foreach c [my children] {
                  if {[lsearch [$c info methods] asXML] ne -1} {
                      $c asXML
                  }
              }
          }
      }

      my instproc remove_all_children_of_type {type} {
          set kids [list]
          foreach child [my children] {
              $child remove_all_children_of_type $type
              if {[$child istype $type]} {
                  my delete $child
              }
          }
      }

      my instproc remove_all_children_ne_type {type} {
          set kids [list]
          foreach child [my children] {
              $child remove_all_children_ne_type $type
              if {![$child istype $type]} {
                  my delete $child
              }
          }
      }

      my instproc all_children_of_type {type} {
          set kids [list]
          foreach child [my children] {
              set kids [concat $kids [$child all_children_of_type $type]]
              if {[$child istype $type]} {
                 lappend kids $child
              }
          }
          return $kids
      }

      my instproc children_of_type {type} {
          set kids [list]
          foreach child [my children] {
              if {[$child istype $type]} {
                  lappend kids $child
              }
          }
          return $kids
      }

      #my instproc destroy args {
      #    if {[my exists dom]} {
      #        # FIXME
      #        [my set dom] delete
      #    }
      #    next
      #}
  }


  XMLClass proc get_valid_id {name} {
    # deprecated
    #package require md5
    #return "ID_[md5::md5 -hex \"$name\"]"
    return [::xoutil::id_generator get_id_for_string $name]
  }

  XMLClass instproc require_tdom_cmd {} {
    namespace eval [self]::xml {
      dom createNodeCmd elementNode [my set tdom_cmd_name]
    }
  }


  XMLClass create TextNode -parameter {
    text
  }

  TextNode instproc asXML {} {
      dom createNodeCmd textNode    t
      t [my text]
  }

}
